import requests 
from typing import List, Dict
import json
from bs4 import BeautifulSoup
import os
from functools import wraps

cookies = {
    'PHPSESSID': 'jm5prvd8ct4iqk7np61mi6pfku',
    '__cf_bm': 'n0g5zqggjWOeyTJzbTzI4ztUZMvJGOjrOePo0qHtupw-1721829708-1.0.1.1-OH.ZB6f1NUaEMAK8Pa0F5dEVjSFMIq8RR3yuDw81DV_sD2lpar_QSwBO74WfBCvykCncfcPZYllg0Qm_.t7uKw',
    'cf_clearance': '7__XQGNPkr3zyafKYLeUKsjC6LQ_bCli28mLvjW2aAA-1721827259-1.0.1.1-KbH.H8Nmkjy0iiXw0AJgZ3fs9ZxHzbXqiSTxqzYYB3WRvrGvn9SeOvQmgkpbsgQJBQ4I0Cpv3CxTutCuo5Nf0A',
    '_ga': 'GA1.2.16528444.1721827261',
    '_gid': 'GA1.2.738335994.1721827262',
    '_gat': '1',
    '__gads': 'ID=652a96df5f1fc503:T=1721827261:RT=1721827261:S=ALNI_Mbjj-prHW9lBxlHvuV5f_sX6Er4pQ',
    '__gpi': 'UID=00000a48e4d9731d:T=1721827261:RT=1721827261:S=ALNI_MY3TK6MsJmfCuivExvJ7pF0Dwdmyg',
    '__eoi': 'ID=449bbecfed5f1702:T=1721827261:RT=1721827261:S=AA-AfjbmPCDQWO2MQRm_ZfO_39R5',
    '_ga_4JSC4GJHB3': 'GS1.2.1721827263.1.0.1721827263.0.0.0',
}
headers = {
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:128.0) Gecko/20100101 Firefox/128.0',
    'Accept': 'application/json, text/javascript, */*; q=0.01',
    'Accept-Language': 'en-US,en;q=0.5',
    'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
    'X-Requested-With': 'XMLHttpRequest',
    'Origin': 'https://freecarrierlookup.com',
    'Connection': 'keep-alive',
    'Referer': 'https://freecarrierlookup.com/',
    'Sec-Fetch-Dest': 'empty',
    'Sec-Fetch-Mode': 'cors',
    'Sec-Fetch-Site': 'same-origin',
    'Priority': 'u=0',
}
def lookup(cc:int=1, phone:int=None, cookies:Dict=cookies, headers:Dict=headers) -> Dict:
    '''Lookup carrier information for a phone number'''
    proxies = {
        'http': os.environ.get('sssmsproxy'),
        'https': os.environ.get('sssmsproxy')
    }
    def parse(html):
        soup = BeautifulSoup(html, 'html.parser')
        return {
                    **{key: soup.find('strong', string=f'{key}:').find_next('p').text for key in ['Phone Number', 'Carrier', 'Is Wireless']},
                    'gateways': {
                        'SMS': soup.find('strong', string='SMS Gateway Address:').find_next('p').text,
                        'MMS': soup.find('strong', string='MMS Gateway Address:').find_next('p').text
                    }
                }
    data = {
                'test': '456Tabo', 
                'sessionlogin': '1', 
                'cc': str(cc),
                'phonenum': str(phone)
            }
    try: 
        response = requests.post('https://freecarrierlookup.com/getcarrier_free.php', headers=headers, cookies=cookies, data=data, proxies=proxies)
        if response.status_code == 200:return parse(response.json()['html'])
    except Exception as e:
        print(response.text)
        raise e


def cache(cachepath:str=os.path.join(os.path.dirname(__file__), 'lookupcache.json')):
    '''Cache lookup results'''
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            if not os.path.exists(cachepath):open(cachepath, 'w').write('{}')
            with open(cachepath, 'r') as f:
                cache = json.load(f)
            if kwargs['phone'] in cache: return cache[kwargs['phone']]
            result = func(*args, **kwargs)
            cache[kwargs['phone']] = result
            with open(cachepath, 'w') as f:
                json.dump(cache, f)
            return result
        return wrapper
    return decorator




'''self-service SMS via email'''
import os
from .lookup import lookup as lkp, cache
from .prx import prx
from . import config
from .sssmtp import SMTP
import smtplib
from email.message import EmailMessage
from rich.console import Console
from rich.progress import Progress
from rich.panel import Panel
import click
import getpass

console = Console()

def safeload():
    try:return config.load()
    except FileNotFoundError:return None

cfg = safeload()

# Apply cache decorator if enabled
if cfg and cfg['cache']['lookups']:lkp = cache(cfg['cache']['path'])(lkp)

@click.group()
@click.option('--noproxy', is_flag=True, default=False, help='Disable proxy')
@click.pass_context
def cli(ctx, noproxy):
    ctx.ensure_object(dict)
    ctx.obj['noproxy'] = noproxy
    
    if cfg is None and not ctx.invoked_subcommand == 'init':
        console.print("Config file not found. Run 'sssms init' to create one.", style="yellow")
        ctx.exit()
    
    if not noproxy and cfg:
        proxy(cfg['proxy']['host'], cfg['proxy']['port'])

@cli.command()
def init():
    """Initialize the configuration file"""
    if cfg:
        console.print("Config file already exists. Use 'load()' to read it or delete the existing file to create a new one.", style="yellow")
        return

    email = input('Please provide the default email to send from: ')
    password = getpass.getpass('Please provide the default email password: ')
    secret = getpass.getpass('Enter a secret key for config encryption: ')

    try:
        config.create(email=email, password=password, secret_key=secret)
        console.print("Config file created successfully.", style="green")
    except Exception as e:
        console.print(f"Failed to create config file: {str(e)}", style="red")


@cli.command()
@click.option('--cc', default=1, help='Country code')
@click.option('--phone', help='Phone number')
def lookup(cc, phone):
    with Progress() as progress:
        task = progress.add_task("[cyan]Looking up...", total=100)
        result = lkp(cc=cc, phone=phone)
        progress.update(task, advance=100)
    console.print(Panel.fit(str(result), title="Lookup Result", border_style="green"))

@cli.command()
@click.argument('message')
@click.option('--cc', default=1, help='Country code')
@click.option('--to', help='Phone number to send message to')
@click.option('--from', 'from_', default=None, help='Email to send message from')
@click.option('--port', default=587, help='SMTP port')
def send(message, cc, to, from_, port):
    if not from_:
        from_ = cfg['credentials']['email']
    
    def asemail(to, message, from_, port):
        msg = EmailMessage()
        msg.set_content(message)
        msg['Subject'] = 'Message'
        msg['From'] = from_
        msg['To'] = lkp(to)['gateways']['SMS']
        server = cfg.get('smtp', {}).get('server') if cfg.get('smtp') else SMTP()(from_)
        try:
            with Progress() as progress:
                task = progress.add_task("[cyan]Sending email...", total=100)
                with smtplib.SMTP(server, port) as server:
                    server.starttls()
                    server.login(from_, cfg['credentials']['password'])
                    progress.update(task, advance=50)
                    server.send_message(msg)
                    progress.update(task, advance=50)
            console.print(Panel("Message sent successfully", title="Success", border_style="green"))
        except Exception as e:
            console.print(Panel(f"Failed to send message: {e}", title="Error", border_style="red"))

    asemail(to, message, from_, port)

@cli.command()
@click.option('--host', default=None, help='Host to run proxy on')
@click.option('--port', default=None, help='Port to run proxy on')
def proxy(host, port):
    host = host or cfg['proxy']['host']
    port = port or cfg['proxy']['port']
    try:
        with Progress() as progress:
            task = progress.add_task("[cyan]Starting proxy...", total=100)
            prx(host, port)
            progress.update(task, advance=100)
        console.print(Panel(f"Proxy running on {host}:{port}", title="Proxy Status", border_style="green"))
    except Exception as e:
        console.print(Panel(f"Failed to start proxy: {e}", title="Error", border_style="red"))

if __name__ == "__main__":
    cli()


import subprocess as sp
import os
from rich.console import Console
console = Console()

def prx(host:str='0.0.0.0', port:int=8899, printout=False):
    '''Start a proxy server'''
    stdout = sp.PIPE if printout else sp.DEVNULL
    sp.Popen(['proxy', '--hostname', host, '--port', str(port)], shell=True, stdout=stdout, stderr=sp.PIPE)
    os.environ['sssmsproxy'] = f'{host}:{port}' if host != '0.0.0.0' else f'localhost:{port}'
    console.print(f'Proxy Running | Addresses: {host} | Port: {port}', style='bold green')

if __name__ == '__main__':
    prx()


import yaml
import os
import platform
from cryptography.fernet import Fernet
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
import base64
import secrets
from pathlib import Path
from .sssmtp import SMTP
import sys

def __key__(secret_key):
    '''Generate encryption key for config file'''
    salt = secrets.token_bytes(16)
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt,
        iterations=100000,
    )
    key = base64.urlsafe_b64encode(kdf.derive(secret_key.encode()))
    return key, base64.b64encode(salt).decode()

def __sysconf__():
    home = Path.home()
    if platform.system() == 'Windows':
        return home / '.bashrc'
    elif 'zsh' in os.environ.get('SHELL', ''):
        return home / '.zshrc'
    else:
        return home / '.bashrc'

def load(path:str=None):
    '''Load configuration from file'''
    if not path:
        path = os.path.join(os.path.dirname(__file__), 'config.yaml')
    if os.path.exists(path):
        with open(path, 'r') as f:
            content = yaml.safe_load(f)
        if 'credentials' in content and 'password' in content['credentials']:
            key = base64.urlsafe_b64decode(content['encryption']['key'])
            content['credentials']['password'] = Fernet(key).decrypt(content['credentials']['password'].encode()).decode()
        return content
    raise FileNotFoundError('Config file not found')

def create(prxhost:str='0.0.0.0', prxport:int=8899, email:str=None, password:str=None, cachelookups:bool=True, cachepath:str=None, server:str=None, secret_key:str=None):
    '''Create a new configuration file'''
    path = os.path.join(os.path.dirname(__file__), 'config.yaml')
    if not cachepath:
        cachepath = os.path.join(os.path.dirname(__file__), 'lookupcache.json')
    if not os.path.exists(path):
        smtp = SMTP()
        if not email or not password or not secret_key:
            raise ValueError("Email, password, and secret key are required")
        if not server:
            try:
                server = smtp(email)
            except ValueError:
                print("Couldn't automatically determine your email provider.")
                print("Supported providers:", ", ".join(smtp.__providers__()))
                provider = input('Please clarify your email provider from the list above, or enter a new SMTP server: ')
                if provider not in smtp.__providers__():
                    smtp.__addnew__(provider, input('Please provide the domain(s) associated with this provider: '))
                if '@' in provider:
                    provider = provider.split('@')[-1].split('.')[0].lower()
                else:
                    try:
                        server = smtp(f'user@{provider}.com')
                    except ValueError:
                        print("Invalid provider. Please try again with a different provider.")
                        sys.exit(1)
        
        key, salt = __key__(secret_key)
        content = {
            'proxy': {
                'host': prxhost,
                'port': prxport
            },
            'credentials': {
                'email': email,
                'password': Fernet(key).encrypt(password.encode()).decode()
            }, 
            'cache': {
                'lookups': cachelookups,
                'path': cachepath
            }, 
            'smtp': {
                'server': server,
                'providers': smtp.common
            },
            'encryption': {
                'key': key.decode(),
                'salt': salt
            }
        }
        with open(path, 'w') as f:
            yaml.dump(content, f)
        print(f"Config file created at {path}")
    else:
        print("Config file already exists. Use 'load()' to read it or delete the existing file to create a new one.")

if __name__ == "__main__":
    create()


from setuptools import setup, find_packages
import os 

with open(f"{os.path.join(os.path.dirname(__file__),'README.md')}", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="sssms",
    version='0.1.7',
    author="Joel Yisrael",
    author_email="joel@sss.bot",
    description="Self-Service SMS via eMail",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/schizoprada/sssms",
    packages=find_packages(),
    install_requires=[
        "click",
        "requests",
        "beautifulsoup4",
        "rich",
        "pyyaml",
        "cryptography",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    entry_points={
        "console_scripts": [
            "sssms=sssms.main:cli",
        ],
    },
)


import yaml
import os

class SMTP:
    '''SMTP server lookup based on email provider'''
    def __init__(self):
        self.config = os.path.join(os.path.dirname(__file__), 'config.yaml')
        self.common = {
                'smtp.gmail.com': ['gmail', 'googlemail'],
                'smtp.mail.yahoo.com': ['yahoo', 'ymail', 'rocketmail'],
                'smtp-mail.outlook.com': ['outlook', 'hotmail', 'live', 'msn'],
                'smtp.aol.com': ['aol', 'aim'],
                'smtp.mail.me.com': ['icloud', 'me', 'mac'],
                'smtp.protonmail.com': ['protonmail', 'pm'],
                'smtp.zoho.com': ['zoho'],
                'smtp.yandex.com': ['yandex'],
                'smtp.gmx.com': ['gmx'],
                'smtp.mail.com': ['mail'],
            }
        self.__config__()

    def __config__(self):
        if os.path.exists(self.config):
            with open(self.config, 'r') as f:
                config = yaml.safe_load(f)
            self.common.update(config.get('smtp', {}).get('providers', {})) if config.get('smtp') else None


    def __save__(self):
        with open(self.config, 'r') as f:
            config = yaml.safe_load(f)
        if 'smtp' not in config:
            config['smtp'] = {}
        config['smtp']['providers'] = self.common
        with open(self.config, 'w') as f:
            yaml.dump(config, f)

    def __addnew__(self, server, domains):
        if isinstance(domains, str):
            domains = [domains]
        if server in self.common:
            self.common[server].extend(domains)
            self.common[server] = list(set(self.common[server]))  
        else:
            self.common[server] = domains
        self.__save__()

    def __remove__(self, server):
        if server in self.common:
            del self.common[server]
            self.__save__()

    def __call__(self, email: str) -> str:
        domain = email.split('@')[-1].split('.')[0].lower()
        for server, domains in self.common.items():
            if domain in domains:
                return server
        raise ValueError('Unsupported email provider')

    def __providers__(self):
        return list(set([domain for domains in self.common.values() for domain in domains]))